require 'acceptance_helpers'
require 'shared/contexts'

resource 'Learning Objects' do

  before { 12.times { create(:learning_object) } }

  let(:learning_objects) { LearningObject.all }
  let(:object_types) { ObjectType.all }
  let(:languages) { Language.all }
  let(:licenses) { License.all }
  let(:subject) { Subject.all }
  let(:educational_stage) { EducationalStage.all }
  let(:tag) { Tag.all }

  get '/v1/learning_objects' do
    parameter :limit, 'Limit of learning objects'
    parameter :offset, 'Offset of learning objects'

    let(:limit) { 12 }
    let(:offset) { 0 }

    example_request 'Get a list of learning objects' do
      # active model serializer may render model associations in different order for collections (array of items), so we're verifing only returned ids
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(LearningObject.limit(limit).offset(offset).pluck(:id).sort)
      expect(status).to eq(200)
    end
  end

  get '/v1/learning_objects/:id' do
    let(:id) { learning_objects.first.id }

    example 'Get a learning object' do
      do_request
      expect(path).to eq("/v1/learning_objects/#{id}") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(LearningObject.find(id)))
      expect(status).to eq(200)
    end
  end

  post '/v1/learning_objects' do
    include_context "authenticate_user"

    parameter :author, 'The author of a educational content', scope: :learning_object
    parameter :name, 'The name of the learning object', scope: :learning_object
    parameter :description, 'The description of educational content of the learning object', scope: :learning_object
    parameter :link, 'Object link, when don’t have any attachments', scope: :learning_object
    parameter :software, 'If some software is needed to run the object', scope: :learning_object
    parameter :tags, 'Array with tags (by name)', scope: :learning_object
    parameter :subjects, 'Array with subjects ids', scope: :learning_object
    parameter :educational_stages, 'Array with educational stages ids', scope: :learning_object
    parameter :object_type_id, 'The id of object type', scope: :learning_object
    parameter :language_id, 'The id of a language', scope: :learning_object
    parameter :license_id, 'The id of a license', scope: :learning_object

    let(:author) { Faker::Name.name }
    let(:name) { Faker::Name.name }
    let(:description) { Faker::Lorem.paragraph }
    let(:object_type_id) { object_types.first.id }
    let(:language_id) { languages.first.id }
    let(:license_id) { licenses.first.id }
    let(:software) { Faker::Lorem::sentence }
    let(:link) { Faker::Internet.url }
    let(:tags) { [ {name: tag.first.name}, {name: tag.last.name} ] }
    let(:subjects) { [ subject.first.id] }
    let(:educational_stages) { [ educational_stage.first.id ] }
    let(:raw_post) { params.to_json }

    example 'Creating a learning_object draft' do
      do_request
      expect(status).to eq(201)
    end
  end

  put '/v1/learning_objects/:id' do
    include_context "authenticate_user"

    parameter :author, 'The author of a educational content'
    parameter :name, 'The name of the learning object'
    parameter :description, 'The description of educational content of the learning object'
    parameter :link, 'Object link, when don’t have any attachments'
    parameter :software, 'If some software is needed to run the object'
    parameter :tags, 'Array with tags (by name)'
    parameter :subjects, 'Array with subjects ids'
    parameter :educational_stages, 'Array with educational stages ids'
    parameter :object_type_id, 'The id of object type'
    parameter :language_id, 'The id of a language'
    parameter :license_id, 'The id of a license'

    #TODO: add items of array
    let(:id) { @learning_object.id }
    let(:description) { Faker::Lorem.paragraph }
    let(:raw_post) { params.to_json }

    before do
      @learning_object = create(:learning_object, publisher: @user)
    end

    example 'Updating a learning_object draft' do
      do_request
      expect(status).to eq(200)
    end
  end

  post '/v1/learning_objects/:id/like' do
    include_context "authenticate_user"

    let(:id) { learning_objects.first.id }

    example 'Liking a learning object' do
      likes = learning_objects.first.likes.count
      do_request
      expect(path).to eq("/v1/learning_objects/#{id}/like")
      expect(JSON.parse(response_body)['count']).to eq(likes + 1)
      expect(status).to eq(201)
    end
  end

  delete '/v1/learning_objects/:id/like' do
    include_context "authenticate_user"

    let(:id) { learning_objects.first.id }

    before do
      create(:like, user: @user, likeable: learning_objects.first)
    end

    example 'Unliking a learning object' do
      likes = learning_objects.first.likes.count
      do_request
      expect(path).to eq("/v1/learning_objects/#{id}/like")
      expect(JSON.parse(response_body)['count']).to eq(likes - 1)
      expect(status).to eq(200)
    end
  end

   get '/v1/learning_objects/:id/versions' do
     include_context "authenticate_user"

     before do
       @learning_object = create(:learning_object)
       @learning_object.update(name: Faker::Name.name)
     end

     let(:id) { @learning_object.id }

     example 'Getting all learning_object versions' do
       do_request
       expect(path).to eq("/v1/learning_objects/#{id}/versions") # `:id` is replaced with the value of `id`
       expect(response_body).to eq(Helper.serialize(@learning_object.versions))
       expect(status).to eq(200)
     end
   end

end
